##Sentinel-1 IW Batch Processing=group
##1 - GRD Products Calibration and Orthorectification over Polygon=name
##Input_Data_Folder=folder
##DEM_Folder=folder
##Input_Polygon_File=vector
##Tile_Field_Name=string NAME
##Path_Field_Name=string path
##Tiles_To_Process=string p120r0-p47r0
##Calibration_Type=string Sigma0
##Output_EPSG=crs
##Output_Resolution=number 10
##Output_Data_Folder=folder
##Ram=number 256


#!/usr/bin/python
#-*- coding: utf-8 -*-
# =========================================================================
# Authors: Cedric Lardeux et Pierre-Louis Frison
# This program take some codes from CESBIO:
#   Program:   S1Processor
#
#   Copyright (c) CESBIO. All rights reserved.
#
#   See LICENSE for details.
#
#   This software is distributed WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the above copyright notices for more information.
#
# =========================================================================
#
# Authors: Thierry KOLECK (CNES)
#
# =========================================================================


import os, shutil, sys, glob, fnmatch
import subprocess, pdb
from osgeo import gdal, ogr, osr
from datetime import datetime, date, time

# INPUT
# Input_Data_Folder = 'F:/cedric/CL/ONFGuyane/Data/Sentinel1/TestScript/Input'
# Output_Data_Folder= 'F:/cedric/CL/ONFGuyane/Data/Sentinel1/TestScript/Ortho'
# Input_Polygon_File = 'F:/cedric/CL/ONFGuyane/Data/Sentinel1/TestScript/Vecteur/NewArea/StudyArea.shp'

# Tiles_To_Process =  'p120r0'

# Tile_Field_Name = 'NAME'
# Path_Field_Name = 'path'

# DEM_Folder='F:/cedric/CL/ONFGuyane/Data/Sentinel1/TestScript/DEM'
# Calibration_Type= 'Sigma0'
# Output_EPSG='EPSG:32622'
# Output_Resolution= 10
# Ram= 2000






Input_Data_Folder = str(Input_Data_Folder)
Output_Data_Folder= str(Output_Data_Folder)
Input_Polygon_File = str(Input_Polygon_File)
DEM_Folder = str(DEM_Folder)

GridShape = str(Input_Polygon_File)
tileNameList = Tiles_To_Process.split('-')
TileField = Tile_Field_Name
PathField = Path_Field_Name

'''
This function list this immediate subdirectories in one given directory
'''
def get_immediate_subdirectories(a_dir):
    return [os.path.join(a_dir, name) for name in os.listdir(a_dir)
            if os.path.isdir(os.path.join(a_dir, name))]

'''
This function list all file with a given file extention in one directory
'''
def directory_liste_ext_files(directory, filt_ext):
   list_file = []
   for root, dirnames, filenames in os.walk(directory):
      for filename in filenames:
         if filename.endswith((filt_ext)):
            list_file.append(os.path.join(root, filename))

   list_file.sort()
   return list_file

def getOrigin(aManifestFile):
   with open(aManifestFile,"r") as saveFile:
      for line in saveFile:
         if "<gml:coordinates>" in line:
            coor = line.replace("                <gml:coordinates>","").replace("</gml:coordinates>","").split(" ")
            coord = [(float(val.replace("\n","").split(",")[0]),float(val.replace("\n","").split(",")[1]))for val in coor]

   return coord[0],coord[1],coord[2],coord[3]


def getRelativeOrbit(aManifestFile):
   file = open(aManifestFile,"r")
   for line in file:
      if "<safe:relativeOrbitNumber type=\"start\">" in line:
         RelativeOrbit = int(line.replace("            <safe:relativeOrbitNumber type=\"start\">","").replace("</safe:relativeOrbitNumber>",""))
   file.close()
   return RelativeOrbit

def getS1ByTile(aInputShape,rawRasterList, aTileFieldName, aPathFieldName, tileNameField):
   tilePath = 0
   aTileFieldName = str(aTileFieldName)
   aPathFieldName = str(aPathFieldName)
   driver = ogr.GetDriverByName("ESRI Shapefile")
   dataSource = driver.Open(aInputShape, 0)
   layer = dataSource.GetLayer()
   # pdb.set_trace()
   for currentTile in layer:
      if str(currentTile.GetField(aTileFieldName))==str(tileNameField):
         tile=currentTile
         tilePath = currentTile.GetField(aPathFieldName)
         break
   poly = ogr.Geometry(ogr.wkbPolygon)
   tileFootPrint = currentTile.GetGeometryRef()
   intersectRaster=[]
   for image in rawRasterList:
      # We get relative orbit
      RasterRelOrbit = getRelativeOrbit(image)
      NW,NE,SE,SW = getOrigin(image)


      poly = ogr.Geometry(ogr.wkbPolygon)
      ring = ogr.Geometry(ogr.wkbLinearRing)
      ring.AddPoint(NW[1], NW[0],0)
      ring.AddPoint(NE[1], NE[0],0)
      ring.AddPoint(SE[1], SE[0],0)
      ring.AddPoint(SW[1], SW[0],0)
      ring.AddPoint(NW[1], NW[0],0)
      poly.AddGeometry(ring)

      intersection = poly.Intersection(tileFootPrint)
      if intersection.GetArea()!=0 and RasterRelOrbit == tilePath:
         intersectRaster.append(image)
      poly = None
      ring = None
   dataSource = None

   return intersectRaster

def getS1IntersectByTile(aInputShape,rawRasterList, aFieldName,tileNameField):
   aFieldName = str(aFieldName)
   # print 'rawRasterList', rawRasterList
   driver = ogr.GetDriverByName("ESRI Shapefile")
   dataSource = driver.Open(aInputShape, 0)
   layer = dataSource.GetLayer()
   for currentTile in layer:
      # print currentTile.GetField('NAME'), tileNameField
      if str(currentTile.GetField(aFieldName))==str(tileNameField):
         tile=currentTile
         break
   poly = ogr.Geometry(ogr.wkbPolygon)
   tileFootPrint = currentTile.GetGeometryRef()
   intersectRaster=[]
   for image in rawRasterList:
      NW,NE,SE,SW = getOrigin(image)

      poly = ogr.Geometry(ogr.wkbPolygon)
      ring = ogr.Geometry(ogr.wkbLinearRing)
      ring.AddPoint(NW[1], NW[0],0)
      ring.AddPoint(NE[1], NE[0],0)
      ring.AddPoint(SE[1], SE[0],0)
      ring.AddPoint(SW[1], SW[0],0)
      ring.AddPoint(NW[1], NW[0],0)
      poly.AddGeometry(ring)

      intersection = poly.Intersection(tileFootPrint)
      if intersection.GetArea()!=0:
         intersectRaster.append(image)

      poly = None
      ring = None

   dataSource = None

   return intersectRaster

def getTileOriginIntersectByS1(GridPath,image):
  import itertools
  manifest = image.getManifest()
  S1FootPrint = getOrigin(manifest)
  poly = ogr.Geometry(ogr.wkbPolygon)
  ring = ogr.Geometry(ogr.wkbLinearRing)
  ring.AddPoint(S1FootPrint[0][1], S1FootPrint[0][0])
  ring.AddPoint(S1FootPrint[1][1], S1FootPrint[1][0])
  ring.AddPoint(S1FootPrint[2][1], S1FootPrint[2][0])
  ring.AddPoint(S1FootPrint[3][1], S1FootPrint[3][0])
  ring.AddPoint(S1FootPrint[0][1], S1FootPrint[0][0])
  poly.AddGeometry(ring)

  driver = ogr.GetDriverByName("ESRI Shapefile")
  dataSource = driver.Open(GridPath, 0)
  layer = dataSource.GetLayer()
   
  intersectTile_tmp = []
  intersectTile = []

  for currentTile in layer:
	 tileFootPrint = currentTile.GetGeometryRef()
	 intersection = poly.Intersection(tileFootPrint)
	 if intersection.GetArea()!=0:
		area_polygon= tileFootPrint.GetGeometryRef(0)
		points=area_polygon.GetPoints()
		intersectTile.append(currentTile.GetField('NAME'))

  dataSource = None

  return intersectTile

def CreateShapeFromTile(aInputShape,aFieldName, aFieldValue, aOutputShape):
   aFieldName = str(aFieldName)
   # print 'rawRasterList', rawRasterList
   driver = ogr.GetDriverByName("ESRI Shapefile")
   dataSource = driver.Open(aInputShape, 0)
   layer = dataSource.GetLayer()
   for currentTile in layer:
      if currentTile.GetField(aFieldName)==aFieldValue:
         tile=currentTile
         break
   tileFootPrint = currentTile.GetGeometryRef()

   # Save extent to a new Shapefile
   outShapefile = aOutputShape
   LayerName = os.path.basename(os.path.splitext(aOutputShape)[0])
   outDriver = ogr.GetDriverByName("ESRI Shapefile")

   # Remove output shapefile if it already exists
   if os.path.exists(outShapefile):
       outDriver.DeleteDataSource(outShapefile)

   # Create the output shapefile
   outDataSource = outDriver.CreateDataSource(outShapefile)
   outLayer = outDataSource.CreateLayer(LayerName, geom_type=ogr.wkbPolygon)

   # Create the feature and set values
   featureDefn = outLayer.GetLayerDefn()
   feature = ogr.Feature(featureDefn)
   feature.SetGeometry(tileFootPrint)
   outLayer.CreateFeature(feature)
   feature = None

   outDataSource = None
   dataSource = None

   srs = osr.SpatialReference()
   srs.ImportFromEPSG(4326)

   OutPrjFile = os.path.splitext(aOutputShape)[0] + '.prj'
   srs.MorphToESRI()
   filePrj = open(OutPrjFile, 'w')
   filePrj.write(srs.ExportToWkt())
   filePrj.close()

def getDateFromS1Folder(aFolderName):
   return aFolderName.split("/")[-1].split("_")[4]

def getDayFromS1File(aFileName):
   aFileName = aFileName.split("/")[-1]
   return aFileName.split("_")[4][0:8]

def getBaseDataNameFromS1Folder(aFolderName):
   return '_'.join(aFolderName.split("/")[-1].split("_")[:5])

def getDateFromS1Raster(PathToRaster):
   return PathToRaster.split("/")[-1].split("-")[4][0:8]

'''
This function calibrate SAR file
aNoise   --> to remove (or not) noise  - True or False
aCalibration_Type    --> to chose aCalibration_Type      - sigma/gamma/beta/dn
'''
def OTBSARCalibration(aInputFile, aOutputFile, aNoise, aCalibration_Type, aRam):
   cmd = "otbcli_SARCalibration -in "
   cmd += aInputFile + " "
   cmd += " -ram " + str(aRam)
   cmd += " -out "
   cmd += aOutputFile
   cmd += " -noise " + aNoise
   cmd += " -lut " + aCalibration_Type

   # progress.setInfo(cmd)
   # print cmd

   p1 = subprocess.Popen (cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,  stderr=subprocess.PIPE)
   ret= p1.communicate()[1]

def GdalClipRasterWithVector(aInputRaster, aInputVector, aOutputFile, aOutputResolution, aSRCNoData, aOutEPSG, aRam):
   cmd = "gdalwarp "
   cmd += "--config GDAL_CACHEMAX " + str(aRam) + " -multi -wo NUM_THREADS=val/ALL_CPUS "
   cmd += " -tr " + str(aOutputResolution) + ' ' + str(aOutputResolution) + ' '
   cmd += " -t_srs " + str(aOutEPSG)
   cmd += " -r average -q -multi -crop_to_cutline -cutline "
   cmd += aInputVector
   cmd += ' -co COMPRESS=DEFLATE -co PREDICTOR=2 '
   cmd += " -dstnodata " + str(aSRCNoData)
   cmd += " -of GTiff " + aInputRaster + ' ' + aOutputFile

   # progress.setInfo(cmd)
   # print cmd

   p1 = subprocess.Popen (cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,  stderr=subprocess.PIPE)
   ret= p1.communicate()[1]

'''
This function do orthorectification of any optical/radar file
aDEMDir  --> directory that contain DEM

opt.gridspacing is not in parameter function but need to be near 4 to 10 times of output pixel size
For example, with 10m output pixel size, you can chose bettween 40 to 100.
More less is the value more accurate is the results but more long
In addition this value have to be linked to DEM pixel size
'''
def OTBOrthorectification(aInputFile, aOutputFile, aDEMDir , aRam):
   cmd = "otbcli_OrthoRectification -io.in "
   cmd += aInputFile + " "
   cmd += " -opt.ram " + str(aRam)
   cmd += " -io.out "
   cmd += aOutputFile
   cmd += " -elev.dem " + aDEMDir
   cmd += " -opt.gridspacing 40 "

   # progress.setInfo(cmd)
   # print cmd

   p1 = subprocess.Popen (cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,  stderr=subprocess.PIPE)
   ret= p1.communicate()[1]

def OTBExtractRoi(aInputFile, aInputRefFile, aOutputFile, aRam):
   cmd = "otbcli_ExtractROI -in "
   cmd += aInputFile + " "
   cmd += " -ram " + str(aRam)
   cmd += " -out "
   cmd += aOutputFile
   cmd += " -mode fit "
   cmd += " -mode.fit.ref " + aInputRefFile

   # progress.setInfo(cmd)
   # print cmd

   p1 = subprocess.Popen (cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,  stderr=subprocess.PIPE)
   ret= p1.communicate()[1]

def ReprojVector(aInputVector, aOutputVector, aEPSG):
   cmd = "ogr2ogr "
   cmd += " -f \"ESRI Shapefile\"  " + aOutputVector+ " "
   cmd +=  aInputVector  + " "
   cmd += " -t_srs EPSG:" + str(aEPSG)

   # print cmd
   p1 = subprocess.Popen (cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,  stderr=subprocess.PIPE)
   ret= p1.communicate()[1]

def Rasterize(aInputVector, aOutputRaster, aOutPixelSize):
   # Multilook
   cmd = "gdal_rasterize "
   cmd += " -burn 1 "
   cmd += " -of GTiff "
   cmd += " -a_nodata 0 "
   cmd += " -a_srs EPSG:3857 "
   cmd += " -tr " + str(aOutPixelSize) + " "+ str(aOutPixelSize)+ " "
   cmd += aInputVector + " "
   cmd += aOutputRaster

   # print cmd
   p1 = subprocess.Popen (cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,  stderr=subprocess.PIPE)
   ret= p1.communicate()[1]

def GRD2Calib_Ortho(aInputFile,aOutputFile, aCalibration_Type, aNoise, aRam):
   TimeNow = datetime.now().strftime("%Y_%m_%d_%H_%M_%S")

   # Output Folder Name
   OutputFolderName = os.path.dirname(aOutputFile)

   # Output Intermediate Calibrated file
   OutputCalibratedFile = os.path.join(TmpDir, TimeNow + "_Calib.tif")

   # Calibration
   OTBSARCalibration(aInputFile, OutputCalibratedFile, aNoise, aCalibration_Type, aRam)

   # Orthorectification
   OTBOrthorectification(OutputCalibratedFile, aOutputFile, DemDir, aRam)

   if os.path.exists(OutputCalibratedFile):
            os.remove(OutputCalibratedFile)

def ProcessS1Dataset(aInputManifestList, aOutputDir, aInputShape, aResolution, aCalibration_Type, aNoise, aEPSG, aRam):
   
   # Rasterize the polygone if enable option
   # Repro Vector to EPSG 3857
   ReprojVectorPath = os.path.join(TmpDir,'ReprojVector.shp')
   ReprojVector(aInputShape, ReprojVectorPath, 3857)
   ClipRasterBase = os.path.join(aOutputDir,'RasterArea.tif')
   Rasterize(ReprojVectorPath, ClipRasterBase, 100)


   # List all SENTINEL-1 sub directories
   S1DataFolders = [ os.path.dirname(ManFile) for ManFile in aInputManifestList]

   # Loop thru different S1 data (folder)
   for Folder in S1DataFolders:
      TiffFiles = directory_liste_ext_files(Folder, 'tiff')

      SafeName = os.path.split(Folder)[-1]
      FolderName = SafeName.split('.')[0]

      OutputFilesOrtho = []
      for TiffFile in TiffFiles:
         OutFolderName = getBaseDataNameFromS1Folder(FolderName) + '_' + str(tile)
         if 'grd-vh' in TiffFile:
            SuffixOrtho = '_VH_'

         if 'grd-hv' in TiffFile:
            SuffixOrtho = '_HV_'
            
         if 'grd-vv' in TiffFile:
            SuffixOrtho = '_VV_'

         if 'grd-hh' in TiffFile:
            SuffixOrtho = '_HH_'

         SuffixOrtho += Calib_Name+'_Ortho.tif'

         OutClipedFile = os.path.join(TmpDir,'S1Clip.tif')
         OTBExtractRoi(TiffFile, ClipRasterBase, OutClipedFile, aRam)
         TiffFile = OutClipedFile

         OutputFile = os.path.join(TmpDir,OutFolderName,OutFolderName + SuffixOrtho)
         if not os.path.exists(os.path.dirname(OutputFile)):
            os.makedirs(os.path.dirname(OutputFile))

         OutputFileCliped = os.path.join(aOutputDir,OutFolderName,OutFolderName + SuffixOrtho)
         if not os.path.exists(os.path.dirname(OutputFileCliped)):
            os.makedirs(os.path.dirname(OutputFileCliped))         

         # Do GRD conversion to Ortho (including calibration)
         GRD2Calib_Ortho(TiffFile,OutputFile, aCalibration_Type, aNoise, aRam)

         GdalClipRasterWithVector(OutputFile,aInputShape , OutputFileCliped, aResolution,  0, aEPSG, aRam)

         if os.path.exists(OutputFile):
            os.remove(OutputFile)



   # Del Temporay Dir
   shutil.rmtree(TmpDir)
   if os.path.exists(ClipRasterBase):
      os.remove(ClipRasterBase)

   # Check if we need to mosaic concecutive data
   # List all folder
   SubFolders = get_immediate_subdirectories(aOutputDir)
   concatenateImage(SubFolders)

def OTBConcatenate(aInputRasterList, aOutputRaster, aRam):
   cmd = "otbcli_BandMath -il "
   for rast in aInputRasterList:
      cmd += rast + ' '
   cmd += " -ram " + str(aRam)
   cmd += " -out "
   cmd += aOutputRaster
   cmd += " -exp "
   cmd += "\"max("
   for i in range(len(aInputRasterList)):
      cmd += 'im' + str(i+1) + 'b1,'
   cmd = cmd[:-1]
   cmd += ")\""

   # print cmd
   p1 = subprocess.Popen (cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,  stderr=subprocess.PIPE)
   ret= p1.communicate()[1]



def concatenateImage(aInputFolderList):
      # MainFolder = 
      FirstFile = aInputFolderList[0]

      MainFolder =  list(os.path.split(FirstFile)[:-1])[0]
      aInputFolderList.sort()

      AcqDateList = [getDateFromS1Folder(os.path.split(FolderName)[-1])[0:8] for FolderName in aInputFolderList]
      UniqueAcqDateList = list(set(AcqDateList))
      for AcDate in UniqueAcqDateList:
         FileToConcatenate = [folder  for folder in aInputFolderList if AcDate in folder]
         if len(FileToConcatenate) >= 2:
            # Create new Directory (same name without hour)
            SFolder = os.path.split(FileToConcatenate[0])
            SFolder = SFolder[-1].split('_')
            SFolder[4] = AcDate
            SFolder = '_'.join(SFolder)
            NewOutDir = os.path.join(MainFolder,SFolder)
            if not os.path.exists(NewOutDir):
               os.makedirs(NewOutDir)

            # List all tiff file
            TiffFiles = []
            for FileToConcat in FileToConcatenate:
               TiffFiles += directory_liste_ext_files(FileToConcat, '.tif')

            # List copol file (VV or HH)
            CopolFile = [ item for item in TiffFiles if ('_VV_' or '_HH_') in item ]
            FileName = os.path.basename(os.path.splitext(CopolFile[0])[0])
            FileName = os.path.split(FileName)
            FileName = FileName[-1].split('_')
            FileName[4] = AcDate
            FileName = '_'.join(FileName)


            OutputConcatCopolFile = os.path.join(NewOutDir,FileName +'.tif')
            # Concatenate Copol
            OTBConcatenate(CopolFile, OutputConcatCopolFile, Ram)

            # List crosspol file (VH or HV)
            CrosspolFile = [ item for item in TiffFiles if ('_VH_' or '_HV_') in item ]
            FileName = os.path.basename(os.path.splitext(CrosspolFile[0])[0])
            FileName = os.path.split(FileName)
            FileName = FileName[-1].split('_')
            FileName[4] = AcDate
            FileName = '_'.join(FileName)


            OutputConcatCrosspolFile = os.path.join(NewOutDir,FileName +'.tif')
            # Concatenate Copol
            OTBConcatenate(CrosspolFile, OutputConcatCrosspolFile, Ram)

            # delete the 2 folders
            for file in CopolFile + CrosspolFile:
               if  os.path.exists(os.path.dirname(file)):
                  shutil.rmtree(os.path.dirname(file))

def GetNewDatesFromListFiles(aInputList, aInputDir, aOutputDir):
   # List all unique date already processed (output folder)
   AllOutputFiles = directory_liste_ext_files(aOutputDir, 'tiff') +  directory_liste_ext_files(aOutputDir, 'tif')
   AllOutputFiles = [ file for file in AllOutputFiles if 'Quegan' not in file]
   # List all unique date in input file
   AllInputFiles = directory_liste_ext_files(aInputDir, 'tiff') +  directory_liste_ext_files(aInputDir, 'tif')
   AllInputFiles = [ file for file in AllInputFiles if 'Quegan' not in file]

   # print '\n\nAllOutputFiles',AllOutputFiles
   # print '\n\nAllInputFiles',AllInputFiles


   ListDates = [getDayFromS1File(RastPath) for RastPath in AllOutputFiles ]
   UniqueOutputDates = list(set(ListDates))
   UniqueOutputDates.sort()
   

   
   ListDates = [getDayFromS1File(RastPath) for RastPath in AllInputFiles ]
   UniqueInputDates = list(set(ListDates))
   UniqueInputDates.sort()
   

   # if len(UniqueOutputDates) == 0:
   #    UniqueOutputDates = ['0']

   # Get New dates
   NewDates = [x for x in UniqueInputDates if x not in UniqueOutputDates]

   # print 'UniqueInputDates, UniqueOutputDates, NewDates', UniqueInputDates, UniqueOutputDates, NewDates

   # Filter the data
   NewList = []
   for NDate in NewDates:
      for item in aInputList:
         if NDate in item:
            NewList.append(item)

   return NewList

def GetNewDatesFromListFilesInputManifest(aInputList, aInputDir, aOutputDir):
   # pdb.set_trace()
   # List all unique date already processed (output folder)
   AllOutputFiles = directory_liste_ext_files(aOutputDir, 'tiff') +  directory_liste_ext_files(aOutputDir, 'tif')
   AllOutputFiles = [ file for file in AllOutputFiles if 'Quegan' not in file]
   # List all unique date in input file
   AllInputFiles = directory_liste_ext_files(aInputDir, 'tiff') +  directory_liste_ext_files(aInputDir, 'tif')
   AllInputFiles = [ file for file in AllInputFiles if 'Quegan' not in file]


   # print '\n\nAllOutputFiles',AllOutputFiles
   # print '\n\nAllInputFiles',AllInputFiles


   ListDates = [getDayFromS1File(RastPath) for RastPath in AllOutputFiles ]
   UniqueOutputDates = list(set(ListDates))
   UniqueOutputDates.sort()
   

   
   ListDates = [getDateFromS1Raster(RastPath) for RastPath in AllInputFiles ]
   UniqueInputDates = list(set(ListDates))
   UniqueInputDates.sort()
   

   # if len(UniqueOutputDates) == 0:
   #    UniqueOutputDates = ['0']

   # Get New dates
   NewDates = [x for x in UniqueInputDates if x not in UniqueOutputDates]

   # print 'UniqueInputDates, UniqueOutputDates, NewDates', UniqueInputDates, UniqueOutputDates, NewDates

   # Filter the data
   NewList = []
   for NDate in NewDates:
      for item in aInputList:
         if NDate in item:
            NewList.append(item)

   return NewList



# Reproject Tile to 4326
GridShapeEPSG4326 = GridShape.replace('.shp', 'EPSG4326.shp')
ReprojVector(GridShape, GridShapeEPSG4326, 4326)


Noise = 'false'
if Calibration_Type == 'Sigma0':
   Calibration_Type = 'sigma'
   Calib_Name='Sig0'
elif Calibration_Type == 'Gamma0':
   Calibration_Type = 'gamma'
   Calib_Name='Gam0'
elif Calibration_Type == 'Beta0':
   Calibration_Type = 'beta'
   Calib_Name='Bet0'
else:
   Calibration_Type='sigma'
   Calib_Name='Sig0'

DemDir = DEM_Folder

# List all SENTINEL-1 manifest.safe files
ManifestFiles = directory_liste_ext_files(Input_Data_Folder, 'manifest.safe')

# Filter the input file to process only new files that are not in Output Folder
ManifestFiles = GetNewDatesFromListFilesInputManifest(ManifestFiles, Input_Data_Folder, Output_Data_Folder)
# pdb.set_trace()
# progress.setInfo('avant tile')
# Process all tiles
for tile in tileNameList:

   # Extract files that intercect the required Tile
   intersectRaster = getS1ByTile(GridShapeEPSG4326,ManifestFiles, TileField, PathField, tile)

   # print intersectRaster
   # pdb.set_trace()

   # Create Output Tile Dir
   TileDir = os.path.join(Output_Data_Folder, str(tile))
   if not os.path.exists(TileDir):
      os.makedirs(TileDir)

   # Create Shape file of the tile
   TileShape = os.path.join(TileDir, str(tile) + '.shp')
   CreateShapeFromTile(GridShapeEPSG4326,TileField, str(tile), TileShape)

   # create tmp dir
   TmpDir = os.path.join(TileDir, "tmp")
   if not os.path.exists(TmpDir):
      os.makedirs(TmpDir)
      
   ProcessS1Dataset(intersectRaster, TileDir, TileShape, Output_Resolution, Calibration_Type, Noise, Output_EPSG, Ram)
